"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Const = require("./Consts");
// @ts-ignore <ts(7016)>
var sha1 = require("js-sha1");
var Evaluator = /** @class */ (function () {
    function Evaluator() {
    }
    Evaluator.prototype.Evaluate = function (_key, _json, _defaultValue, _user) {
        // @ts-ignore <ts(2339)>
        var flags = Object.values(_json);
        // @ts-ignore <ts(2769)>
        var flag = flags.find(function (flag) { return flag.key === _key; });
        if (flag === undefined || flag === null) {
            // No flag found, return default value
            return _defaultValue;
        }
        if (_user === undefined || _user === null) {
            // If flag is rollout but no user is present to evaluate,
            // return the default value
            if (flag['is_rollout']) {
                return _defaultValue;
            }
            return flag['value'];
        }
        if (!flag['is_targeting_enabled']) {
            // Evaluate rollouts
            if (flag['is_rollout']) {
                return this.EvaluateRollout(_key, _user['id'], flag['rollouts'], _defaultValue);
            }
            return flag['value'];
        }
        if (flag['is_targeting_enabled']) {
            return this.EvaluateTargets(_key, flag['targets'], _user, _defaultValue);
        }
        return _defaultValue;
    };
    Evaluator.prototype.EvaluateTargets = function (_key, _targets, _user, _defaultValue) {
        // @ts-ignore <ts(2339)>
        var targets = Object.values(_targets);
        var value = _defaultValue;
        var ruleResult = false;
        var evaluates = true;
        var target;
        for (var _i = 0, targets_1 = targets; _i < targets_1.length; _i++) {
            target = targets_1[_i];
            value = target['value'];
            if (target['rules']) {
                ruleResult = this.EvaluateRules(target['rules'], _user);
                evaluates = evaluates && ruleResult;
            }
            if (evaluates) {
                if (target['is_rollout']) {
                    return this.EvaluateRollout(_key, _user['id'], target['rollouts'], _defaultValue);
                }
                return value;
            }
        }
        return _defaultValue;
    };
    Evaluator.prototype.EvaluateRules = function (_rules, _user) {
        // @ts-ignore <ts(2339)>
        var rules = Object.values(_rules);
        var result = false;
        var rule;
        for (var _i = 0, rules_1 = rules; _i < rules_1.length; _i++) {
            rule = rules_1[_i];
            var userAttributeValue = this.GetUserAttributeValue(rule['attribute'], _user);
            if (!userAttributeValue)
                continue;
            var comparator = rule['comparator'];
            var values = rule['values'];
            switch (comparator) {
                case Const.COMPARATOR_EQUAL_TO:
                    result = values.includes(userAttributeValue);
                    break;
                case Const.COMPARATOR_NOT_EQUAL_TO:
                    result = !values.includes(userAttributeValue);
                    break;
                case Const.COMPARATOR_CONTAINS:
                    for (var _a = 0, values_1 = values; _a < values_1.length; _a++) {
                        var value = values_1[_a];
                        result = userAttributeValue.includes(value);
                        if (result)
                            break;
                    }
                    break;
                case Const.COMPARATOR_NOT_CONTAIN:
                    var valid = true;
                    for (var _b = 0, values_2 = values; _b < values_2.length; _b++) {
                        var value = values_2[_b];
                        result = !userAttributeValue.includes(value);
                        valid = result && valid;
                    }
                    result = valid;
                    break;
                default:
                    result = false;
                    break;
            }
        }
        return result;
    };
    Evaluator.prototype.EvaluateRollout = function (_key, _userId, _rollouts, _defaultValue) {
        var rolloutValue = _defaultValue;
        var rolloutLowerLimit = 0;
        var rolloutUpperLimit = 0;
        var scale = this.GetScale(_key, _userId);
        for (var _i = 0, _rollouts_1 = _rollouts; _i < _rollouts_1.length; _i++) {
            var rollout = _rollouts_1[_i];
            rolloutUpperLimit = rolloutUpperLimit + parseInt(rollout['percentage']);
            if (scale > rolloutLowerLimit && scale <= rolloutUpperLimit) {
                rolloutValue = rollout['value'];
                break;
            }
            rolloutLowerLimit = rolloutLowerLimit + parseInt(rollout['percentage']);
        }
        return rolloutValue;
    };
    Evaluator.prototype.GetScale = function (_key, _userId) {
        var hashString = _key + _userId;
        var hash = sha1.hex(hashString).substring(0, 7);
        var scale = parseInt(hash, 16) % 100;
        return scale;
    };
    Evaluator.prototype.GetUserAttributeValue = function (_key, _user) {
        if (_key.toLowerCase() === "email") {
            return _user.email;
        }
        if (_user.custom && _user.custom[_key.toLowerCase()].toLowerCase()) {
            return _user.custom[_key];
        }
        return null;
    };
    return Evaluator;
}());
exports.Evaluator = Evaluator;
